/**
 * This is the template of a class that evaluates a given policy,
 * i.e., computes the utility of each state when actions are chosen
 * according to it.  The utility is returned in the public
 * <tt>utility</tt> field.  You need to fill in the constructor.  You
 * may wish to add other fields with other useful information that 
 * you want this class to return (for instance, number of iterations
 * before convergence).
 */
public class PolicyEval {

    /** the computed utility of each state under the given policy */
    public double utility[];

    /** the precision used to determine when to stop iterating (called
     * epsilon in lecture) **/
    public static double precision = 1e-10;

    /**
     * The constructor for this class.  Computes the utility of policy
     * <tt>pi</tt> for the given <tt>mdp</tt> with given
     * <tt>discount</tt> factor, and stores the answer in
     * <tt>utility</tt>.
     */
    public PolicyEval(Mdp mdp, double discount, int pi[]) {

	// your code here

    }

}
